%GENSC
%
%  Generalized seam carving for image retargeting. Using the default settings inside
%  this script, results are produced by our improved seam carving algorithm.
%
%  [imCurr imShift imVisible] = gensc(inputfile,S,boolFigures,boolFiles)
%          
%  inputfile   string   image directory
%  S           int      number of seams to remove
%  boolFigures boolean  show figures at each iteration? (default 0)
%  boolFiles   boolean  write files at end? (default 0)
%
%  imCurr      uint8    output image
%  imShift     double   output horizontal shift map
%  imVisible   uint8    output visibility map
%
%  For more information see the paper 'Visibility Maps for Improving Seam Carving',
%  Image Retargeting Workshop, European Conference on Computer Vision (ECCV) 2010 
%  available at http://www.vision.ee.ethz.ch/~mansfiea/improvingsc/
% 
%  Copyright (C) 2011 Alex Mansfield, alex.mansfield@vision.ee.ethz.ch

function [imCurr imShift imVisible] = gensc(inputfile,S,boolFigures,boolFiles)

%    ---------------------------------------------------------------------
%
%    MATLAB/MEX Generalized seam carving      
%    Copyright (C) 2011 Alex Mansfield (alex.mansfield@vision.ee.ethz.ch)
%
%    This program is free software: you can redistribute it and/or modify
%    it under the terms of the GNU General Public License as published by
%    the Free Software Foundation, either version 3 of the License, or
%    (at your option) any later version.
%
%    This program is distributed in the hope that it will be useful,
%    but WITHOUT ANY WARRANTY; without even the implied warranty of
%    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
%    GNU General Public License for more details.
%
%    You should have received a copy of the GNU General Public License
%    along with this program.  If not, see http://www.gnu.org/licenses/.
%
%    ---------------------------------------------------------------------

s = RandStream.create('mt19937ar','seed',0);
RandStream.setDefaultStream(s);

% Settings ---------------------------------------------------------------

% Note that the parameters chosen for 'improved seam carving' in our paper are
% highlighted with a *

% Image
nameImage = inputfile;

if( nargin<3 || isempty(boolFigures) )
    % Plot figures?
    boolFigures = 0;
end

if( nargin<4 || isempty(boolFiles) )
    % Write figures to files?
    boolFiles = 0;
end

% Pairwise energy function
% 0 - absolute difference in magnitude *
% 1 - norm of RGB difference
% 2 - shift map energy
for pairwise = [0];

% Pairwise term order
% 1 - default *
for pairwiseorder = [1];

% Unary energy function
% 0 - No unary
% 1 - SC backwards energy *
unary = 1;

% Unary order
% 1 - default *
for unaryorder = [1];

% Unary weight
% 1 - default * 
for unaryweight = [1];

% Input unary terms?
% 0 - do not use *
% 1 - use
unaryinput = 0; 

% Which shift term?
% 0 - none
% 1 - additive square shift term
% 2 - count energy for number of seams between
% 3 - unary based
% 4 - unary based with averaging
% 5 - 4 + 2 for diagonal terms *
for shiftterm = [5];

% Shift weight
% 1 - default * 
for shiftweight = [1];

% Use refinement steps?
% 0 - no refinement *
% 1 - refinement
for refinement = [0];
    
% Pixel blend ratio
% 0 - no blending *
for blend = [0];

% Use a total energy?
% 0 - No
% 1 - Yes *
for total = [1]
    
% Use a log speedup?
% 0 - No *
% 1 - Simple
for logspeedup = [0]
    
    %disp(nameImage);  
    %disp(['Pairwise ' int2str(pairwise) ' Pairwise Order ' num2str(pairwiseorder)]);
    %disp(['Unary ' int2str(unary) ' Unary Order ' num2str(unaryorder) ' Unary Weight ' num2str(unaryweight) ' Unary Input ' int2str(unaryinput)]);
    %disp(['Shift term ' int2str(shiftterm) ' Shift weight ' num2str(shiftweight)]);
    %disp(['Refinement ' int2str(refinement) ' Blend ' num2str(blend) ' Total ' int2str(total)]);
    %disp(['Log speedup ' int2str(logspeedup)]);

filepreImage = pwd;
filepostImage = ['/' inputfile];

% Timings file
%fileTime = '/scratch/mansfiea/ClassifyKillSC/scvtimings';
fileTime = [pwd '/timings'];
 
% Constants
infinity = 100000000;
un_small = infinity;

% Rules -------------------------------------------------------------------

% cannot do refinement without a total energy
if(total==0)
    refinement = 0;
end

% Functions ---------------------------------------------------------------

fncsPairwise = {@pairwise0,@pairwise1,@pairwise2};
fncsShift = {@NULL,@shift1,@shift2,@shift3,@shift4,@shift5};
fncsShiftOpH = {@NULL,@plus,@times,@plus,@plus,@plus};
fncsShiftOpV = {@NULL,@plus,@times,@NULL,@NULL,@times};

fncPairwise = fncsPairwise{pairwise+1};
fncShift = fncsShift{shiftterm+1};
fncShiftOpV = fncsShiftOpV{shiftterm+1};
fncShiftOpH = fncsShiftOpH{shiftterm+1};

% Run optimization --------------------------------------------------------

tic
    [imCurr imShift imVisible] = seamcarve_mex(filepreImage,filepostImage,nameImage,fileTime,infinity,un_small,S,pairwise,pairwiseorder,unary,unaryorder,unaryweight,unaryinput,shiftterm,shiftweight,refinement,boolFigures,boolFiles,blend,total,logspeedup,fncPairwise,fncShift,fncShiftOpV,fncShiftOpH);
toc

end
end
end
end
end
end
end
end
end
end