
/*    ---------------------------------------------------------------------
 *
 *    MATLAB/MEX Generalized seam carving
 *    Copyright (C) 2011 Alex Mansfield (alex.mansfield@vision.ee.ethz.ch)
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 *    ---------------------------------------------------------------------
 */ 

#include "sc.h"

#include <stdlib.h>
#include <stdio.h>
#include <assert.h>

int sc_seamcarve(int const R, int const C, int const pairwise, double const pairwiseorder, int const unary, double const unaryorder, double const unaryweight, int const unaryinput, int const shiftterm, double const shiftweight, int const refinement, double const blend, int const total, int const logspeedup, double const infinity, double const un_small, int const S, absPairwise const * const fncPairwise, absShift const * const fncShift, absOp const * const fncShiftOpV, absOp const * const fncShiftOpH, double const * const imInput, double const * const imInputMagnitude, double const * const imInputGradient, double const * const imInputUnary, double const * const imInputEnergy, double const * const imInputInstance, double * const imCurrOut, double * const imShiftOut, double * const imVisibleOut, double * const totalDistortionOut, double * const imEnergyOut) {

  double * const imCurr = (double*)malloc(R*C*3*sizeof(double));
  double * const imMagnitude = (double*)malloc(R*C*sizeof(double));
  double * const imGradient = (double*)malloc(R*C*sizeof(double));
  double * const imUnary = (double*)malloc(R*C*sizeof(double));
  double * const imEnergy = (double*)malloc(R*C*3*sizeof(double));
  double * const imInstance = (double*)malloc(R*C*sizeof(double));
  int * const imShift = (int*)malloc(R*C*sizeof(int));
  int * const imVisible = (int*)malloc(R*C*sizeof(int));
  int * const imNumber = (int*)malloc(R*C*sizeof(int));
  int * const imBound = (int*)malloc(R*C*sizeof(int));
  double * const imDistortion = (double*)malloc(R*C*2*sizeof(double));
  int * const vecSeam = (int*)malloc(R*sizeof(int));
    
  for(int r=0; r<R; r++) {
    vecSeam[r] = 0;
    for(int c=0; c<C; c++) {
      imMagnitude[index2(r,c,R)] = imInputMagnitude[index2(r,c,R)];
      imGradient[index2(r,c,R)] = imInputGradient[index2(r,c,R)];
      imUnary[index2(r,c,R)] = imInputUnary[index2(r,c,R)];
      imInstance[index2(r,c,R)] = imInputInstance[index2(r,c,R)];
      imShift[index2(r,c,R)] = 0;
      imVisible[index2(r,c,R)] = 1;
      imNumber[index2(r,c,R)] = c;
      if(c==1)
	imBound[index2(r,c,R)] = C;
      else
	imBound[index2(r,c,R)] = 0;
      for(int i=0; i<3; i++) {
	imCurr[index3(r,c,i,R,C)] = imInput[index3(r,c,i,R,C)];
	imEnergy[index3(r,c,i,R,C)] = imInputEnergy[index3(r,c,i,R,C)];
	if(i<2)
	  imDistortion[index3(r,c,i,R,C)] = 0.0;
      }
    }
  }

  int bound_max = 1;
  int now_crop = 0;
  int currC = C;

  srand(0);

  for(int it=0; it<S; it++) {
    //optimization
    sc_optimize(imEnergy,currC,C,R,now_crop,un_small,infinity,unaryinput,logspeedup,imInstance,imBound,bound_max,vecSeam);

    //calculate imDistortion
    sc_distortion_remove(R,currC,C,vecSeam,imEnergy,fncPairwise,pairwiseorder,blend,imCurr,imMagnitude,imInput,imInputMagnitude,imShift,imNumber,fncShift,fncShiftOpV,fncShiftOpH,shiftweight,imInputUnary,it,imDistortion);

    //remove seam
    sc_seam_remove(vecSeam,unaryinput,refinement,blend,logspeedup,imCurr,imMagnitude,imEnergy,imShift,imInstance,imDistortion,currC,C,R,imUnary,imVisible,imNumber);

    //update energy
    sc_energy(R,currC,C,it,vecSeam,2,1,un_small,infinity,unary,unaryinput,total,imDistortion,imUnary,imInstance,fncPairwise,pairwiseorder,blend,imCurr,imMagnitude,imInput,imInputMagnitude,imInputUnary,imShift,imNumber,fncShift,fncShiftOpV,fncShiftOpH,shiftweight,imEnergy);

    if(now_crop==1) {
      for(int r=0; r<R; r++) {
	if(imInstance[index2(r,0,R)]>0)
	  imEnergy[index3(r,0,1,R,C)] = un_small;
	if(imInstance[index2(r,currC-1,R)]>0)
	  imEnergy[index3(r,currC-1,1,R,C)] = un_small;
      }
    }
  }

  //calculate totalDistortion
  double totalDistortion = 0.0;
  for(int r=0; r<R; r++) {
    for(int c=0; c<currC; c++) {
      for(int i=0; i<2; i++) {
	totalDistortion += imDistortion[index3(r,c,i,R,C)];
      }
      if(imVisible[index2(r,c,R)]==0)
	totalDistortion += imInputUnary[index2(r,c,R)];
    }
    for(int c=currC; c<C; c++) {
      if(imVisible[index2(r,c,R)]==0)
	totalDistortion += imInputUnary[index2(r,c,R)];
    }
  }

  // assign outputs

  assert(currC==(C-S));

  for(int r=0; r<R; r++) {
    for(int c=0; c<currC; c++) {
      imShiftOut[index2(r,c,R)] = (double)imShift[index2(r,c,R)];
      imVisibleOut[index2(r,c,R)] = (double)imVisible[index2(r,c,R)];
      for(int i=0; i<3; i++) {
	imCurrOut[index3(r,c,i,R,currC)] = imCurr[index3(r,c,i,R,C)];
	imEnergyOut[index3(r,c,i,R,currC)] = imEnergy[index3(r,c,i,R,C)];
      }
    }
    for(int c=currC; c<C; c++)
      imVisibleOut[index2(r,c,R)] = (double)imVisible[index2(r,c,R)];
  }
  *totalDistortionOut = totalDistortion;

  // free temporary dynamically assigned arrays

  free(imCurr);
  free(imMagnitude);
  free(imGradient);
  free(imUnary);
  free(imEnergy);
  free(imInstance);
  free(imShift);
  free(imVisible);
  free(imNumber);
  free(imBound);
  free(imDistortion);
  free(vecSeam);
  
  return 1;

}
