
/*    ---------------------------------------------------------------------
 *
 *    MATLAB/MEX Generalized seam carving
 *    Copyright (C) 2011 Alex Mansfield (alex.mansfield@vision.ee.ethz.ch)
 *    This program is free software: you can redistribute it and/or modify
 *    it under the terms of the GNU General Public License as published by
 *    the Free Software Foundation, either version 3 of the License, or
 *    (at your option) any later version.
 *
 *    This program is distributed in the hope that it will be useful,
 *    but WITHOUT ANY WARRANTY; without even the implied warranty of
 *    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
 *    GNU General Public License for more details.
 *
 *    You should have received a copy of the GNU General Public License
 *    along with this program.  If not, see http://www.gnu.org/licenses/.
 *
 *    ---------------------------------------------------------------------
 */ 

/* function seamcarve_inside_mex.cpp
 * Compile:
 * mex seamcarve_inside_mex.cpp sc_inline.cpp
 */
#include "mex.h"
#include <matrix.h>
#include <string.h>
#include <assert.h>

#include "sc.h"

void mexFunction(int nlhs, mxArray *plhs[],int nrhs, mxArray const *prhs[])
{

#ifndef NDEBUG
  if(!(nrhs==25))
    mexErrMsgTxt("Needs 25 inputs!");
  if((nlhs<4))
    mexErrMsgTxt("Needs at least 4 outputs!");
#endif

  // Inputs ---------------------------------------------------------------

  // assert types
  assert(mxIsDouble(prhs[0]));
  assert(mxIsDouble(prhs[1]));
  assert(mxIsDouble(prhs[2]));
  assert(mxIsDouble(prhs[3]));
  assert(mxIsDouble(prhs[4]));
  assert(mxIsDouble(prhs[5]));
  assert(mxIsDouble(prhs[6]));
  assert(mxIsDouble(prhs[7]));
  assert(mxIsDouble(prhs[8]));
  assert(mxIsDouble(prhs[9]));
  assert(mxIsDouble(prhs[10]));
  assert(mxIsDouble(prhs[11]));
  assert(mxIsDouble(prhs[12]));
  assert(mxIsDouble(prhs[13]));
  assert(mxIsDouble(prhs[14]));
  assert(mxIsChar(prhs[15]));
  assert(mxIsChar(prhs[16]));
  assert(mxIsChar(prhs[17]));
  assert(mxIsChar(prhs[18]));
  assert(mxIsDouble(prhs[19]));
  assert(mxIsDouble(prhs[20]));
  assert(mxIsDouble(prhs[21]));
  assert(mxIsDouble(prhs[22]));
  assert(mxIsDouble(prhs[23]));
  assert(mxIsDouble(prhs[24]));

  // get pointers to inputs
  int const pairwise             = (int)*(double*)mxGetData(prhs[0]);
  double const pairwiseorder     = *(double*)mxGetData(prhs[1]);
  int const unary                = (int)*(double*)mxGetData(prhs[2]);
  double const unaryorder        = *(double*)mxGetData(prhs[3]);
  double const unaryweight       = *(double*)mxGetData(prhs[4]);
  int const unaryinput           = (int)*(double*)mxGetData(prhs[5]);
  int const shiftterm            = (int)*(double*)mxGetData(prhs[6]);
  double const shiftweight       = *(double*)mxGetData(prhs[7]);
  int const refinement           = (int)*(double*)mxGetData(prhs[8]);
  double const blend             = *(double*)mxGetData(prhs[9]);
  int const total                = (int)*(double*)mxGetData(prhs[10]);
  int const logspeedup           = (int)*(double*)mxGetData(prhs[11]);
  double const infinity          = *(double*)mxGetData(prhs[12]);
  double const un_small          = *(double*)mxGetData(prhs[13]);
  int const S                    = (int)*(double*)mxGetData(prhs[14]);
  char * fncPairwiseIn           = mxArrayToString(prhs[15]); //allocates dynamic memory
  char * fncShiftIn              = mxArrayToString(prhs[16]); //allocates dynamic memory
  char * fncShiftOpVIn           = mxArrayToString(prhs[17]); //allocates dynamic memory
  char * fncShiftOpHIn           = mxArrayToString(prhs[18]); //allocates dynamic memory
  double * const imInput         = (double*)mxGetData(prhs[19]);
  double * const imMagnitude     = (double*)mxGetData(prhs[20]);
  double * const imGradient      = (double*)mxGetData(prhs[21]);
  double * const imUnary         = (double*)mxGetData(prhs[22]);
  double * const imEnergy        = (double*)mxGetData(prhs[23]);
  double * const imInstance      = (double*)mxGetData(prhs[24]);

  // input sizes
  mwSize const * const size = mxGetDimensions(prhs[19]);
  int const R = (int)size[0];
  int const C = (int)size[1];

  // setup functors
  absPairwise * fncPairwise;
  absShift * fncShift;
  absOp * fncShiftOpV, * fncShiftOpH;

  if(strcmp(fncPairwiseIn,"pairwise0")==0) {
    fncPairwise = new cntPairwise0; //allocates dynamic memory
  } else if(strcmp(fncPairwiseIn,"pairwise1")==0) {
    fncPairwise = new cntPairwise1; //allocates dynamic memory
  } else if(strcmp(fncPairwiseIn,"pairwise2")==0) {
    fncPairwise = new cntPairwise2; //allocates dynamic memory
  }

  if(strcmp(fncShiftIn,"NULL")==0) {
    fncShift = new cntShiftNULL; //allocates dynamic memory
  } else if(strcmp(fncShiftIn,"shift1")==0) {
    fncShift = new cntShift1; //allocates dynamic memory
  } else if(strcmp(fncShiftIn,"shift2")==0) {
    fncShift = new cntShift2; //allocates dynamic memory
  } else if(strcmp(fncShiftIn,"shift3")==0) {
    fncShift = new cntShift3; //allocates dynamic memory
  } else if(strcmp(fncShiftIn,"shift4")==0) {
    fncShift = new cntShift4; //allocates dynamic memory
  } else if(strcmp(fncShiftIn,"shift5")==0) {
    fncShift = new cntShift5; //allocates dynamic memory
  }

  if(strcmp(fncShiftOpVIn,"NULL")==0) {
    fncShiftOpV = new cntOpNULL; //allocates dynamic memory
  } else if(strcmp(fncShiftOpVIn,"plus")==0) {
    fncShiftOpV = new cntOpPlus; //allocates dynamic memory
  } else if(strcmp(fncShiftOpVIn,"times")==0) {
    fncShiftOpV = new cntOpTimes; //allocates dynamic memory
  }

  if(strcmp(fncShiftOpHIn,"NULL")==0) {
    fncShiftOpH = new cntOpNULL; //allocates dynamic memory
  } else if(strcmp(fncShiftOpHIn,"plus")==0) {
    fncShiftOpH = new cntOpPlus; //allocates dynamic memory
  } else if(strcmp(fncShiftOpHIn,"times")==0) {
    fncShiftOpH = new cntOpTimes; //allocates dynamic memory
  } 

  // Outputs --------------------------------------------------------------

  // allocate outputs
  mwSize sizeOutput[3];
  sizeOutput[0] = R;
  sizeOutput[1] = C-S;
  sizeOutput[2] = 3;
  plhs[0] = mxCreateNumericArray(3,sizeOutput,mxDOUBLE_CLASS,mxREAL); //imCurr
  plhs[1] = mxCreateDoubleMatrix(R,C-S,mxREAL);                       //imShift
  plhs[2] = mxCreateDoubleMatrix(R,C,mxREAL);                         //imVisible
  plhs[3] = mxCreateDoubleScalar(0.0);                                //totalDistortion
  plhs[4] = mxCreateNumericArray(3,sizeOutput,mxDOUBLE_CLASS,mxREAL); //imEnergy

  // check allocation
#ifndef NDEBUG
  if(plhs[0]==NULL)
    mexErrMsgTxt("Cannot create output pointer 0");
  if(plhs[1]==NULL)
    mexErrMsgTxt("Cannot create output pointer 1");
  if(plhs[2]==NULL)
    mexErrMsgTxt("Cannot create output pointer 2");
  if(plhs[3]==NULL)
    mexErrMsgTxt("Cannot create output pointer 3");
  if(plhs[4]==NULL)
    mexErrMsgTxt("Cannot create output pointer 4");
#endif

  double * const imCurr = mxGetPr(plhs[0]);
  double * const imShift = mxGetPr(plhs[1]);
  double * const imVisible = mxGetPr(plhs[2]);
  double * const totalDistortion = mxGetPr(plhs[3]);
  double * const imEnergyOut = mxGetPr(plhs[4]);

  // Run c++ seam carving -------------------------------------------------

  if(!sc_seamcarve(R,C,pairwise,pairwiseorder,unary,unaryorder,unaryweight,unaryinput,shiftterm,shiftweight,refinement,blend,total,logspeedup,infinity,un_small,S,fncPairwise,fncShift,fncShiftOpV,fncShiftOpH,imInput,imMagnitude,imGradient,imUnary,imEnergy,imInstance,imCurr,imShift,imVisible,totalDistortion,imEnergyOut))
    mexErrMsgTxt("Error running sc_seamcarve");

  // Free dynamically allocated memory ------------------------------------
  
  mxFree(fncPairwiseIn);
  mxFree(fncShiftIn);
  mxFree(fncShiftOpVIn);
  mxFree(fncShiftOpHIn);

  delete fncPairwise;
  delete fncShift;
  delete fncShiftOpV;
  delete fncShiftOpH;

  return;
} 
